"use strict";

let Url = require('url-parse');
let request = require('request');
let crypto = require('crypto');
let jobQueue =  require('./jobQueue');
let queueNames = require('./queueNames');

class Base{
  constructor(request, config, appName){
    this.request = request;
    this.config = config;
    this.appName = this.getValidAppName(appName);
    this.apiSecret = config.communicator.signingKey;
    this.queueNames = queueNames;
    this.domain = null;
    this.queue = jobQueue.PractoQueue.getInstance(
      config.communicator.communicatorQueueUri,
      config.communicator.sqsSpoolDir);
    try {
      if (request.headers.host.indexOf('http')===-1) {
        if(request.headers.host.indexOf('.local') > -1) {
          this.domain = 'http://' + request.headers.host;
        }
        else{
          this.domain = 'https://' + request.headers.host;
        }
      } else{
        this.domain = request.headers.host;
      }
      this.queue.setDefaultHost(this.domain);
    }
    catch(err) {}
  }

  setDomain(domain=null){
    this.domain = domain;
    this.queue.setDefaultHost(this.domain);
  }

  getDomain() {
    return this.domain;
  }

  getCommunicatorUri(){
    domain = this.domain;
    let parts = new Url(domain);
    const host = parts.hostname;
    const hostParts = host.split('.', 1);
    let subDomain = hostParts[0];
    subDomain = subDomain.split('-');
    let uri = parts.protocol + '//communicator';
    if(subDomain.length > 1){
      uri = uri + '-' + subDomain[1];
    }
    uri = uri + '.' + host.substr(host.indexOf('.')+1);
    return uri;
  }

  generateNonce(){
    let currentTime = (+new Date)/1000;
    const base = "abcdefghijklmnopqrstuvwxyz0123456789";
    let ret = parseInt(currentTime).toString() + '|';
    for(let i=0; i<20; i++){
      ret = ret + base[Math.floor(Math.random() * (base.length - 0) + 0)];
    }
    return ret;
  }

  get(endPoint, requestParams, callback){
    let nonce = this.generateNonce();
    let authToken = crypto.createHash('sha256').update(
        this.apiSecret + nonce.toString()).digest('hex');
    requestParams.appName = this.appName;
    const apiBaseUrl = this.getCommunicatorUri();
    const apiUrl = apiBaseUrl + endPoint.toString();
    const options = {
      url: apiUrl,
      qs: requestParams,
      method: 'GET',
      headers: {
        'nonce': nonce,
        'X-AUTH-TOKEN': authToken
      }
    }
    request(options, function(err, response, body){
      if(err){
        callback(err);
      }
      else{
        callback(null, response);
      }
    });
  }

  post(endPoint, requestParams, callback){
    let nonce = this.generateNonce();
    let authToken = crypto.createHash('sha256').update(
        this.apiSecret + nonce.toString()).digest('hex');
    requestParams.appName = this.appName;
    const apiBaseUrl = this.getCommunicatorUri();
    const apiUrl = apiBaseUrl + endPoint.toString();
    const options = {
      url: apiUrl,
      body: requestParams,
      method: 'POST',
      headers: {
        'nonce': nonce,
        'X-AUTH-TOKEN': authToken
      }
    }
    request(options, function(err, response, body){
      if(err){
        callback(err);
      }
      else{
        callback(null, response);
      }
    });
  }

  send(data, queueName, delay){
    this.queue.insertJob(JSON.stringify(data), queueName, delay=delay);
  }

  getValidAppName(appName){
    const apps = ["RAY", "HELLO", "FABRIC", "EPICENTER", "ACCOUNTS", "STORE",
      "DX-NEO", "HOLONET", "GENESIS", "ONENESS", "TODO", "ADDON"];
    appName = appName.toUpperCase();
    if(apps.indexOf(appName) > -1){
      return appName;
    }
    else{
      throw "Invalid application name. Has to be one of \
            RAY, HELLO, FABRIC, EPICENTER, ACCOUNTS, STORE, \
            DX-NEO, HOLONET, GENESIS, ONENESS";
    }
  }
}

module.exports = {
  Base: Base
}
